% function [Ki1,K1,Kd1,Ki2,K2,Kd2,b0,b1,b2,b3,b4,a0,a1,a2,a3,a4] = area2PID_filter_r (num,den,Tdelay,Tf);
%
% Function araa2PID calculates parameters of the PID controller, tuned for disturbance rejection:
%
% u = (Ki2/s + K2 + Kd2*s) * (y-rf)
%
% from the measured areas of the process A0 to A5 (A0 is the process steady-state gain)
% Signal rf is filtered signal of the reference (see below).
% The parameter Kmax represents the highest allowed open-loop gain K*A0 
% There is a filter
%
% Gf = (b0 + b1*s + b2*s^2 + b3*s^3)/(a0 + a1*s + a2*s^2 + a3*s^3)
%
% on the reference:
% 
% RF(s) = R(s)*Gf(s)

function [Ki1,K1,Kd1,Ki2,K2,Kd2,b0,b1,b2,b3,b4,a0,a1,a2,a3,a4] = area2PID_filter_r (num,den,Tdelay,Tf);

den1 = conv(den,[Tf, 1]);

[A0,A1,A2,A3,A4,A5] = numden2areas (num,den1,Tdelay);
[Ki1,K1,Kd1] = area2PID (A0,A1,A2,A3,A4,A5,100);

[Ki2, K2, Kd2] = PID_Opt_Dist (num,den,Tdelay,Tf);

b0 = Ki1*Ki2*A0;
b1 = Ki1*(K2*A0+1-Ki2*A1)+K1*Ki2*A0;
b2 = Ki1*(-K2*A1+Kd2*A0+Ki2*A2)+K1*(K2*A0+1-Ki2*A1)+Kd1*Ki2*A0;
b3 = Ki1*(K2*A2-Kd2*A1-Ki2*A3)+K1*(-K2*A1+Kd2*A0+Ki2*A2)+Kd1*(K2*A0+1-Ki2*A1);
b4 = Ki1*(Kd2*A2-K2*A3+Ki2*A4)+K1*(K2*A2-Kd2*A1-Ki2*A3)+Kd1*(-K2*A1+Kd2*A0+Ki2*A2);

a0 = Ki1*Ki2*A0;
a1 = Ki2*(K1*A0+1-Ki1*A1)+K2*Ki1*A0;
a2 = Ki2*(-K1*A1+Kd1*A0+Ki1*A2)+K2*(K1*A0+1-Ki1*A1)+Kd2*Ki1*A0;
a3 = Ki2*(K1*A2-Kd1*A1-Ki1*A3)+K2*(-K1*A1+Kd1*A0+Ki1*A2)+Kd2*(K1*A0+1-Ki1*A1);
a4 = Ki2*(Kd1*A2-K1*A3+Ki1*A4)+K2*(K1*A2-Kd1*A1-Ki1*A3)+Kd2*(-K1*A1+Kd1*A0+Ki1*A2);
